/*
SRB2Kart Arcade Mode
AKA Daytona Kart

Made by Aquatic (Lion Sora), Auro (Latius) and LunarRay

Current Version: Version 1.3.2

Credits to the following:
supercomputer276 for the Wario Land 4 Timer Font
Latius for the custom 7-segment numbers used and for the Wario Land 4 Clock
Afrorule for their Start Boost Helper which has been modified to work with the Daytona HUD
LunarRay and MrNin for improving the Map HELL Time Limits
LunarRay for the new voices and improved time limits for the base maps
Snu for their Hitfeed mod, which this mod now has intergrations with
Callmore for their HP mod, which this mod now has intergrations with
*/

local version = "Version 1.4.2"

freeslot(
"sfx_arccr", "sfx_arclo", "sfx_arcko", "sfx_arcbt",
"sfx_am1lpr", "sfx_am2lpr", "sfx_am5lpr", "sfx_am7lpr",
"sfx_amatuw", "sfx_amgoal", "sfx_amlot1", "sfx_amlot2",
"sfx_amsc1", "sfx_amsc2", "sfx_amsc3", "sfx_amscgo",
"sfx_amtex1", "sfx_amtex2", "sfx_amtex3", "sfx_amtex4", "sfx_amtex5"
)

local MILESPERHOUR = 1
local KILOMETERSPERHOUR = 2
local NAUTICALMILESPERHOUR = 3
local METERSPERSECOND = 4
local FOOTPERSECOND = 5
local FRACUNITSPERTIC = 6

local TAHUD = -1
local DAYTONAHUD = 1
local WARIOLANDHUD = 2
local BATTLEMANIAHUD = 3

//Debug stuff
local dkartdebug = {
	enabled = false,
	hp = 0,
	regen = 0,
	latent = 0
}

local arcadeon = CV_RegisterVar({"arcade", "On", CV_NETVAR|CV_SHOWMODIF, CV_OnOff})

local difficulty = CV_RegisterVar({"arcade_diff", "Normal", CV_NETVAR|CV_SHOWMODIF, {Easiest=30, Easy=15, Normal=0, Hard=-5, Expert=-10, Master=-20}})
local sneakeronly = CV_RegisterVar({"arcade_sneaker", "Off", CV_NETVAR|CV_SHOWMODIF, CV_OnOff})
local customspeedometer = CV_RegisterVar({"arcade_speed", "Miles", CV_SHOWMODIF, {Disabled=0, Miles=MILESPERHOUR, Kilometers=KILOMETERSPERHOUR, Meters=METERSPERSECOND, Fracunits=FRACUNITSPERTIC, Knots=NAUTICALMILESPERHOUR, Feet=FOOTPERSECOND}})
local tenthspeedometer = CV_RegisterVar({"arcade_tenthspeed", "On", CV_SHOWMODIF, CV_OnOff})
local timercap = CV_RegisterVar({"arcade_capto99", "Off", CV_SHOWMODIF, {Off = 0, On = 1, ID = 2}})
local leadingzero = CV_RegisterVar({"arcade_placezero", "Off", CV_SHOWMODIF, CV_OnOff})
local instaupdate = CV_RegisterVar({"arcade_instupdate", "On", CV_SHOWMODIF, CV_OnOff})
local hudtype = CV_RegisterVar({"arcade_hud", "Daytona", CV_SHOWMODIF, {Daytona = DAYTONAHUD, WarioLand = WARIOLANDHUD, BattleMania = BATTLEMANIAHUD}})
local hpmodtimeover = CV_RegisterVar({"arcade_hpmod_timeover", "Off", CV_NETVAR|CV_SHOWMODIF, CV_OnOff})
local timeoverpercent = CV_RegisterVar({"arcade_hpmod_topercent", "Off", CV_NETVAR, CV_OnOff})
local timeoverrate = CV_RegisterVar{
	name = "arcade_hpmod_totic",
	defaultvalue = 5,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 1, MAX = TICRATE}
}
local timeoverdam = CV_RegisterVar{
	name = "arcade_hpmod_torate",
	defaultvalue = 1,
	flags = CV_NETVAR,
	PossibleValue = {MIN = 1, MAX = 100}
}
local timeoverdiffflag = CV_RegisterVar{
	name = "arcade_hpmod_diffmod",
	defaultvalue = "Off",
	flags = CV_NETVAR|CV_SHOWMODIF,
	PossibleValue = CV_OnOff
}

local timeoverdiffdefault = {
	[-20] = 4,  //1% damage every 4 Frames
	[-10] = 5,  //1% damage every 5 Frames
	[-5]  = 10, //1% damage every 10 Frames
	[0]   = 17, //1% damage every 17 frames (about 0.5 seconds)
	[15]  = 25, //1% damage every 25 Frames
	[30]  = 35  //1% damage every second 
}

rawset(_G, "daytonakart", {
	active = arcadeon.value,
	hud = hudtype.value,
	diff = difficulty.value,
	timeover = hpmodtimeover.value,
	topercent = timeoverpercent.value,
	torate = timeoverrate.value,
	todam = timeoverdam.value
})

local FLASHTIME = 5 * TICRATE / 2
local BASETIME = 120
local BASEBONUS = 30
local TIMEOVERLENGTH = 5 * TICRATE

local REDTEXT = 1
local GREENTEXT = 2
local YELLOWTEXT = 3

local TIMEFORTHREE = 26 + (TICRATE * 3)
local TIMEFORTWO = 26 + (TICRATE * 4)
local TIMEFORONE = 26 + (TICRATE * 5)
local TIMEFORGO = 26 + (TICRATE * 6)

local UPDATESPEED = 2

local endtimer

local flash = {
	[0]	= 655360, //Fully transparent
	[1] = V_90TRANS,
	[2] = V_80TRANS,
	[3] = V_70TRANS,
	[4] = V_60TRANS,
	[5] = V_50TRANS,
	[6] = V_40TRANS,
	[7] = V_30TRANS,
	[8] = V_20TRANS,
	[9] = V_10TRANS,
	[10] = 0 //Fully opaque
}

//Used to set the base start and bonus times for the base maps, and give a default if none is set
local maptimes = setmetatable({
	//Base Maps
    [1] =   {75, 30}, //Green Hills Zone 
	[2] =   {100, 45}, 
	[3] =   {90, 45}, 
	[4] =   {75, 30}, 
	[5] =   {80, 35}, 
	[6] =   {80, 35}, 
	[7] =   {85, 40}, 
	[8] =   {75, 30}, 
	[9] =   {90, 45}, 
	[10] =  {100, 60}, 
	[11] =  {75, 30}, 
	[12] =  {80, 35},  
	[13] =  {80, 35}, 
	[14] =  {75, 30}, 
	[15] =  {80, 35}, 
	[16] =  {105, 65}, 
	[17] =  {95, 50}, 
	[18] =  {80, 35}, 
	[19] =  {95, 50}, 
	[20] =  {105, 65}, 
	[21] =  {75, 25}, 
	[22] =  {90, 50}, 
	[23] =  {85, 40}, 
	[24] =  {95, 50}, 
	[25] =  {100, 60}, 
	[26] =  {75, 20}, 
	[27] =  {90, 45}, 
	[28] =  {90, 45}, 
	[29] =  {95, 50}, 
	[30] =  {100, 60},
	[31] =  {90, 45}, 
	[32] =  {95, 50}, 
	[33] =  {85, 40}, 
	[34] =  {80, 35}, 
	[35] =  {100, 55}, 
	[36] =  {75, 30}, 
	[37] =  {80, 35}, 
	[38] =  {95, 50}, 
	[39] =  {90, 45}, 
	[40] =  {100, 60}, 
	[41] =  {70, 15}, 
	[42] =  {75, 25}, 
	[43] =  {100, 60}, 
	[44] =  {70, 20}, 
	[45] =  {95, 55}, 
	[46] =  {70, 20}, 
	[47] =  {70, 15}, 
	[48] =  {75, 30}, 
	[49] =  {70, 25}, 
	[50] =  {80, 35}, 
	[51] =  {70, 15}, 
	[52] =  {70, 15}, 
	[53] =  {65, 10}, 
	[54] =  {80, 25}, 
	[55] =  {70, 15}, 
	
	///Community Maps
	[56] =  {90, 45}, //Lake Margorite Zone
	[57] =  {95, 55}, //Coastal Temple Zone
	[58] =  {80, 35}, //Kart Airlines Zone
	[59] =  {90, 45}, //Honeybee Industry Zone
	[60] =  {80, 35}, //Crimson Core Zone
	
    //MAP HELL Maps
    [352] = {130, 90}, //Crystal Abyss Zone 
    [353] = {75, 30},
    [354] = {70, 25}, 
    [355] = {90, 45}, //VV Classic 
    [356] = {115, 75}, 
    [357] = {90, 45}, 
    [358] = {70, 25}, 
    [359] = {75, 25}, 
    [360] = {85, 40}, //DS Classic 
    [361] = {85, 40}, 
    [362] = {75, 25}, 
    [363] = {80, 30}, 
    [364] = {80, 35},  //FZ Silence 
    [365] = {75, 20}, 
    [369] = {70, 25}, //BM Classic
}, {__index = function() return {BASETIME, BASEBONUS} end})

local timeextension = {
{sfx_amtex1, 90},
{sfx_amtex2, 79},
{sfx_amtex3, 64},
{sfx_amtex4, 72},
{sfx_amtex5, 52}
}

local ITEMCHANGE = {
	-1, //Sneaker
	-1, //Rocket Sneaker
	-1, //Invincability
	1, //Banana
	1, //Buttnic Monitor
	1, //Orbinaut
	1, //Jawz
	1, //Mine
	1, //Ballhog
	2, //Blue Bomb
	2, //Grow
	2, //Shrink
	-1, //Thunder Shield
	-1, //Hyudoro
	-1, //Pogo Spring
	-1 //Kitchen Sink
}

//The base Max Speed for all 9 base speed types 
local basemaxspeed = {
	2519263, //Speed 1
	2568660, //Speed 2
	2618057, //Speed 3
	2667455, //Speed 4
	2716852, //Speed 5
	2766249, //Speed 6
	2815647, //Speed 7
	2865044, //Speed 8
	2914441  //Speed 9
}

//The Max RPM (outside of boosting) for all 9 base weight types. Can be adjusted
local baseRPM = {
	7000, //Weight 1
	7125, //Weight 2
	7250, //Weight 3
	7375, //Weight 4
	7500, //Weight 5
	7625, //Weight 6
	7750, //Weight 7
	7875, //Weight 8
	8000  //Weight 9
}


local function settimelimit(map)
	local timelimit
	local basetimelimit
	local lapmod
	local estimatedlaptime
	local bonus
	
	local mapset = maptimes[map]
	
	//Get the Base Time Limit
	if tonumber(mapheaderinfo[map].arcadetime) ~= nil then //Check if the map has a custom time limit set
		basetimelimit = tonumber(mapheaderinfo[map].arcadetime) //Use that time
		if basetimelimit > 120 then basetimelimit = 120 end //Base time can't be more than 120s
		if basetimelimit < 30 then basetimelimit = 30 end   //And must be at least 30s
	else
		basetimelimit = mapset[1]
	end
	
	//Modify the Time Limit depending on the Difficulty
	if daytonakart.diff > 0 then //Easy Mode gives a mod to the timelimit. You get at least 5s more on Easy, and 10s more on Easiest.
		timelimit = (basetimelimit * (100 + daytonakart.diff)) / 50
		if (timelimit % 10) > 4 then //Round it to a multiple of 5s
			timelimit = $ + 10 - (timelimit % 10)
		else
			timelimit = $ - (timelimit % 10)
		end 
		timelimit = $ / 2
		if basetimelimit == timelimit then timelimit = basetimelimit + 5 end
		if (basetimelimit + 5) == timelimit and daytonakart.diff == 30 then timelimit = basetimelimit + 10 end
	elseif daytonakart.diff < 0 then //Hard, Expert and Master gives a starting time penalty
		timelimit = basetimelimit + daytonakart.diff //As difficulty.value is negative, this reduces the time limit
		if basetimelimit < 15 then basetimelimit = 15 end //But never lower than 15s
	else
		timelimit = basetimelimit
	end
	//print("Starting Time: "..timelimit.."s")
	
	//Modify the Time Limit depending on the Number of Laps compared to the default Number of Laps
	if tonumber(mapheaderinfo[map].arcadebonus) ~= nil then //Check if the map has a custom bonus set
		bonus = tonumber(mapheaderinfo[map].arcadebonus) //Use that time
		if bonus < 10 then bonus = 10 end //Bonus must be at least 10s
		if bonus > 90 then bonus = 90 end //And at most 90s
		if tonumber(mapheaderinfo[map].arcadetime) ~= nil then
			if bonus > tonumber(mapheaderinfo[map].arcadetime) then
				bonus = tonumber(mapheaderinfo[map].arcadetime) //Also, you can't get more time from a time extension than you start with
			end
		elseif bonus > BASETIME then
			bonus = BASETIME //Same here
		end
	else
		bonus = mapset[2]
	end
	
	if daytonakart.diff > 0 then //Easy/Easiest Mode gives a mod to the bonus time
		bonus = ($ * (100 + daytonakart.diff)) / 50
		if (bonus % 2) == 1 then bonus = $ + 1 end
		bonus = $ / 2
	elseif daytonakart.diff < 0 then //Hard and Expert reduces the bonus
		bonus = $ + max((daytonakart.diff / 2), -5) //Hard reduces it by 2s, Expert and Master by 5s
		if bonus < 5 then bonus = 5 end //But to no fewer than 5 seconds
	end
	//print("Bonus Time per Lap: "..bonus.."s")
	
	estimatedlaptime = (timelimit + (bonus * (mapheaderinfo[map].numlaps - 1))) / mapheaderinfo[map].numlaps
	if ((estimatedlaptime % 5) > 0) then //Round it up to a multiple of 5s
		estimatedlaptime = $ + 5 - (estimatedlaptime % 5)
	end 
	if estimatedlaptime < 10 then estimatedlaptime = 10 end //Make sure estimated lap times are at least 10 seconds
	//print("Estimated Lap Time: "..estimatedlaptime.."s")
	//print("Mod per Lap: "..(estimatedlaptime - bonus).."s")
	
	lapmod = numlaps
	lapmod = $ - mapheaderinfo[map].numlaps //Postive if numlaps is more than mapheaderinfo[map].numlaps
	//print("Number of Laps: "..numlaps)
	//print("Original Laps: "..mapheaderinfo[map].numlaps)
	//print("lapmod: "..lapmod)
	
	timelimit = $ + (lapmod * (estimatedlaptime - bonus))
	if timelimit < 15 then timelimit = 15 end
		
	return timelimit * TICRATE
end

local function bonustime(map)
	local bonus
	
	if tonumber(mapheaderinfo[map].arcadebonus) ~= nil then //Check if the map has a custom bonus set
		bonus = tonumber(mapheaderinfo[map].arcadebonus) //Use that time
		if bonus < 10 then bonus = 10 end //Bonus must be at least 10s
		if bonus > 90 then bonus = 90 end //And at most 90s
		if tonumber(mapheaderinfo[map].arcadetime) ~= nil then
			if bonus > tonumber(mapheaderinfo[map].arcadetime) then
				bonus = tonumber(mapheaderinfo[map].arcadetime) //Also, you can't get more time from a time extension than you start with
			end
		elseif bonus > BASETIME then
			bonus = BASETIME //Same here
		end
	else
		local mapset = maptimes[map]
		bonus = mapset[2]
	end
	
	//Modify the Bonus depending on the Difficulty
	if daytonakart.diff > 0 then //Easy/Easiest Mode gives a mod to the bonus time
		bonus = ($ * (100 + daytonakart.diff)) / 50
		if (bonus % 2) == 1 then bonus = $ + 1 end
		bonus = $ / 2
	elseif daytonakart.diff < 0 then //Hard and Expert reduces the bonus
		bonus = $ + (daytonakart.diff / 2) //Hard reduces it by 2s, Expert by 5s
		if bonus < 5 then bonus = 5 end //But to no fewer than 5 seconds
	end
	
	return bonus * TICRATE
end

local function startset(player, map)
	//Input: The player, and the Map Number
	//Doesn't return anything, but sets the player.arcade values
	if player.arcade == nil then player.arcade = {} end
	player.arcade.timelimit = settimelimit(map) //Default time limit
	if player.realtime > 0 then player.realtime = 0 end //Reset the timer to the start.
	player.arcade.extendflash = 0 //Used to flash the Time Extension image and also for the TIMEOVER screen
	player.arcade.lapcheck = 0 //Used to check if the lap had changed
	player.arcade.timeover = false //Used to show if time had run out
	player.arcade.mo = nil //Used for the death mobj
	player.arcade.bonustime = bonustime(map)
	player.arcade.quirplength = 0 //Used to delay lap notifications
	player.arcade.numlaps = 0 //Used for if there are multiple 'laps' per lap
	if player.realtime > player.arcade.timelimit then player.realtime = 0 end //Gives new players a .realtime of 0 if time had otherwise expired
	player.arcade.timetoshow = player.arcade.timelimit / TICRATE 
	player.arcade.wariotime = player.arcade.timelimit / TICRATE //Wario Time works slightly different to normal time
	player.arcade.timeincrease = 0 // Used to extend the time limit a la Initial D
	player.arcade.teamtimebonus = false //Did a teammate extend the time for us?
	player.arcade.hpcapped = false //Has HP been capped yet?
	if hpmod and hpmod.running then
		if daytonakart.hud == WARIOLANDHUD then
			hpmod_showbar = false --We use our own HP bar, thanks!
		else
			hpmod_showbar = true --We don't have our own HP bar, so use the default one.
		end
		player.arcade.hpcap = hpmod.maxhp
	end
	if mapheaderinfo[map].noextension then
		player.arcade.timelimit = $ + ((numlaps - 1) * player.arcade.bonustime)
		player.arcade.timetoshow = player.arcade.timelimit / TICRATE 
		player.arcade.wariotime = player.arcade.timelimit / TICRATE
	end
end

local function drawLetter(v, font, char, x, y, f, c)
	local asc = string.byte(char)
	local padding = "000"
	if asc > 99
		padding = ""
	elseif asc > 9
		padding = "0"
	elseif asc > 0
		padding = "00"
	end
	v.draw(x, y, v.cachePatch(font .. padding .. tostring(asc)), f, c)
end

local function drawLetterScaled(v, font, char, x, y, scale, f, c)
	local asc = string.byte(char)
	local padding = "000"
	if asc > 99
		padding = ""
	elseif asc > 9
		padding = "0"
	elseif asc > 0
		padding = "00"
	end
	v.drawScaled(x, y, scale, v.cachePatch(font .. padding .. tostring(asc)), f, c)
end

local function drawManiaNum(v, num, x, y, f, colflag)
	local font = "R_NUM_"
	if colflag == REDTEXT then font = "R"..$
	elseif colflag == GREENTEXT then font = "G"..$ end
	v.draw(x, y, v.cachePatch(font .. num), f)
end

local function drawManiaTime(v, time, x, y, f, colflag)
	local str = tostring(time)
	local leftx = x - (string.len(str) * 7)
	for a=1,string.len(str) do
		drawManiaNum(v, string.sub(str, a, a), leftx + ((a - 1) * 14), y, f, colflag)
	end
end

local function drawFontString(v, font, str, x, y, spacing, f, c)
	for a=1,string.len(str) do
		drawLetter(v, font, string.sub(str, a, a), x + ((a - 1) * spacing), y, f, c)
	end
end

local function drawScaledFontString(v, font, str, x, y, spacing, scale, f, c)
	for a=1,string.len(str) do
		drawLetterScaled(v, font, string.sub(str, a, a), x + ((a - 1) * spacing), y, scale, f, c)
	end
end

local function drawMKtext(v, str, x, y, flags)
	if str == nil
	or string.len(str) == 0
		str = "NIL"
	end
	str = tostring(str)
	str = str:upper()
	drawFontString(v, "MKFNT", str, x, y, 12, flags)
end

local function drawSmallWarioLandNumber(v, num, x, y, flags)
	if num == nil
	or string.len(num) == 0
		num = "0"
	end
	num = tostring(num)
	drawScaledFontString(v, "WTMR", num, x*FRACUNIT, y*FRACUNIT, 9*FRACUNIT, FRACUNIT/2, flags) 
end

//Ported over from the Sneakerless Trial Edition
local taTimes = {
	//Gold Time, Silver Time, Bronze Time (all in Tics)
	// {g*TICRATE, s*TICRATE, b*TICRATE}
	[1]   = {100*TICRATE, 110*TICRATE, 135*TICRATE},
	[2]   = {150*TICRATE, 160*TICRATE, 190*TICRATE},
	[3]   = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[4]   = {110*TICRATE, 120*TICRATE, 135*TICRATE},
	[5]   = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[6]   = {130*TICRATE, 140*TICRATE, 150*TICRATE},
	[7]   = {130*TICRATE, 140*TICRATE, 165*TICRATE},
	[8]   = {100*TICRATE, 110*TICRATE, 135*TICRATE},
	[9]   = {160*TICRATE, 170*TICRATE, 180*TICRATE},
	[10]  = {130*TICRATE, 140*TICRATE, 160*TICRATE},
	[11]  = {110*TICRATE, 120*TICRATE, 135*TICRATE},
	[12]  = {120*TICRATE, 130*TICRATE, 140*TICRATE},
	[13]  = {125*TICRATE, 135*TICRATE, 140*TICRATE},
	[14]  = {110*TICRATE, 120*TICRATE, 145*TICRATE},
	[15]  = {110*TICRATE, 120*TICRATE, 160*TICRATE},
	[16]  = {150*TICRATE, 160*TICRATE, 170*TICRATE},
	[17]  = {160*TICRATE, 170*TICRATE, 195*TICRATE},
	[18]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[19]  = {160*TICRATE, 170*TICRATE, 195*TICRATE},
	[20]  = {135*TICRATE, 145*TICRATE, 170*TICRATE},
	[21]  = {130*TICRATE, 140*TICRATE, 150*TICRATE},
	[22]  = {110*TICRATE, 120*TICRATE, 140*TICRATE},
	[23]  = {140*TICRATE, 150*TICRATE, 165*TICRATE},
	[24]  = {160*TICRATE, 170*TICRATE, 190*TICRATE},
	[25]  = {140*TICRATE, 150*TICRATE, 160*TICRATE},
	[26]  = {110*TICRATE, 120*TICRATE, 145*TICRATE},
	[27]  = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[28]  = {140*TICRATE, 150*TICRATE, 180*TICRATE},
	[29]  = {160*TICRATE, 170*TICRATE, 195*TICRATE},
	[30]  = {125*TICRATE, 135*TICRATE, 160*TICRATE},
	[31]  = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[32]  = {140*TICRATE, 150*TICRATE, 195*TICRATE},
	[33]  = {140*TICRATE, 150*TICRATE, 165*TICRATE},
	[34]  = {110*TICRATE, 120*TICRATE, 150*TICRATE},
	[35]  = {155*TICRATE, 165*TICRATE, 210*TICRATE},
	[36]  = {100*TICRATE, 110*TICRATE, 135*TICRATE},
	[37]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[38]  = {155*TICRATE, 165*TICRATE, 195*TICRATE},
	[39]  = {135*TICRATE, 145*TICRATE, 180*TICRATE},
	[40]  = {110*TICRATE, 120*TICRATE, 160*TICRATE},
	[41]  = {90*TICRATE,  100*TICRATE, 130*TICRATE},
	[42]  = {100*TICRATE, 110*TICRATE, 150*TICRATE},
	[43]  = {110*TICRATE, 120*TICRATE, 160*TICRATE},
	[44]  = {100*TICRATE, 110*TICRATE, 130*TICRATE},
	[45]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[46]  = {100*TICRATE, 110*TICRATE, 130*TICRATE},
	[47]  = {110*TICRATE, 120*TICRATE, 130*TICRATE},
	[48]  = {100*TICRATE, 110*TICRATE, 135*TICRATE},
	[49]  = {95*TICRATE,  105*TICRATE, 120*TICRATE},
	[50]  = {110*TICRATE, 120*TICRATE, 150*TICRATE},
	[51]  = {90*TICRATE,  100*TICRATE, 130*TICRATE},
	[52]  = {95*TICRATE,  105*TICRATE, 130*TICRATE},
	[53]  = {80*TICRATE,  90*TICRATE,  105*TICRATE},
	[54]  = {140*TICRATE, 150*TICRATE, 180*TICRATE},
	[55]  = {80*TICRATE,  90*TICRATE,  130*TICRATE},
	[56]  = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[57]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[58]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[59]  = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[60]  = {120*TICRATE, 130*TICRATE, 150*TICRATE},

	[352] = {190*TICRATE, 200*TICRATE, 220*TICRATE},
	[353] = {105*TICRATE, 115*TICRATE, 135*TICRATE},
	[354] = {100*TICRATE, 110*TICRATE, 120*TICRATE},
	[355] = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[356] = {160*TICRATE, 170*TICRATE, 190*TICRATE},
	[357] = {150*TICRATE, 160*TICRATE, 180*TICRATE},
	[358] = {100*TICRATE, 110*TICRATE, 125*TICRATE},
	[359] = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[360] = {140*TICRATE, 150*TICRATE, 165*TICRATE},
	[361] = {140*TICRATE, 150*TICRATE, 165*TICRATE},
	[362] = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[363] = {140*TICRATE, 150*TICRATE, 170*TICRATE},
	[364] = {120*TICRATE, 130*TICRATE, 150*TICRATE},
	[365] = {125*TICRATE, 135*TICRATE, 155*TICRATE},
	[369] = {100*TICRATE, 110*TICRATE, 120*TICRATE}
}


local function padTimerZero(num)
	if num < 10
		return "0" .. tostring(num)
	else
		return tostring(num)
	end
end

local function drawTimer(v, str, xmid, y, flags, color)
	if str == nil
	or string.len(str) == 0
		str = "0"
	end
	str = tostring(str)
	str = str:upper()
	if string.len(str) == 1 then str = "0" .. $ end
	local x = xmid
	x = $ - (string.len(str) * 9)
	local font = "TMER"
	if color == REDTEXT then font = $ .. "R" end
	if color == GREENTEXT then font = $ .. "G" end
	if color == YELLOWTEXT then font = $ .. "Y" end
	drawFontString(v, font, str, x, y, 18, flags)
end

local function drawTAMedalTimes(v, p, x, y, flags)
	if not (p and p.valid) then return end
	if not taTimes[gamemap] then return end
	local timestep = false
	for i = 0, 2 do
		if taTimes[gamemap][i+1] then
			local medalpatch = ({
				[0] = v.cachePatch("GOTITA"),
				[1] = v.cachePatch("GOTITB"),
				[2] = v.cachePatch("GOTITF")
			}) [i]
			local time = taTimes[gamemap][i+1]
			local str = padTimerZero(G_TicsToMinutes(time, true)) .. ":" .. padTimerZero(G_TicsToSeconds(time)) .. "." ..  padTimerZero(G_TicsToCentiseconds(time))
			local colflag = 0
			if (not timestep) and (time > p.realtime) then 
				if p.exiting then colflag = V_GREENMAP else colflag = V_YELLOWMAP end
				timestep = true
			end
			v.drawScaled(x*FRACUNIT, (y + (10*i))*FRACUNIT, FRACUNIT/2, medalpatch, flags)
			if p.realtime >= time then 
				local fflags = flags&(~V_HUDTRANS)
				fflags = $|V_HUDTRANSHALF
				colflag = V_GRAYMAP
				v.drawScaled(x*FRACUNIT, (y + (10*i))*FRACUNIT, FRACUNIT/2, v.cachePatch("NEEDIT"), fflags)
			end
			v.drawString(x+10, y + (10*i), str, flags|colflag)
		end
	end
end

local function drawTATimer(v, p, timertime, xmid, y, flags)
	if not (p and p.valid) then return end
	local littletime = nil
	local littletimestr = "--:--.--"
	local medal = 4
	local bigtimecolflag = 0
	local littletimecolflag = V_GRAYMAP
	if taTimes[gamemap] then
		for i = 1, 3
			if p.realtime < taTimes[gamemap][i] then
				littletime = taTimes[gamemap][i] - p.realtime
				littletimecolflag = 0
				littletimestr = padTimerZero(G_TicsToMinutes(littletime, true)) .. ":" .. padTimerZero(G_TicsToSeconds(littletime)) .. "." ..  padTimerZero(G_TicsToCentiseconds(littletime))
				medal = i
				break
			end
		end
	end
	local medalpatch = ({
		[1] = v.cachePatch("GOTITA"),
		[2] = v.cachePatch("GOTITB"),
		[3] = v.cachePatch("GOTITF"),
		[4] = v.cachePatch("NEEDIT")
	}) [medal]
	
	if p.exiting then
		bigtimecolflag = YELLOWTEXT
		if littletimecolflag == 0 then littletimecolflag = V_GREENMAP end
	elseif p.arcade.extendflash > 0 and (p.arcade.extendflash % 8) < 4 then
		bigtimecolflag = GREENTEXT
	elseif (timertime < 10) and ((p.realtime % 8) >= 4) and (leveltime >= TIMEFORONE) then
		bigtimecolflag = REDTEXT
	end
	v.drawString(xmid, y-8, "TIME LIMIT", flags, "center")
	drawTimer(v, timertime, xmid, y, flags, bigtimecolflag)
	if littletimecolflag == 0 then
		if (littletime < 10*TICRATE) and ((p.realtime % 8) >= 4) then
			littletimecolflag = V_REDMAP
		elseif (littletime >= 10*TICRATE) and (littletime <= 30*TICRATE) and ((p.realtime % 16) >= 8) then
			littletimecolflag = V_YELLOWMAP
		end
	end
	v.drawString(xmid, y+28, littletimestr, flags|littletimecolflag, "center")
	v.drawScaled((xmid + (4*string.len(littletimestr)))*FRACUNIT, (y+28)*FRACUNIT, FRACUNIT/2, medalpatch, flags)
	
end


local function drawWarioLandTimer(v, p, timer, xmid, y, flags)
	if not (p and p.valid) then return end
	local timermins
	local timersecs
	local timercolon = ((p.realtime % TICRATE) < (TICRATE / 2) and ":" or " ") 
	local timerstring
	local fontstring = "WTMR"
	
	if timer > 59 then
		timermins = tostring(timer / 60)
		timersecs = padTimerZero(timer % 60)
	else
		timermins = "0"
		timersecs = padTimerZero(timer)
	end
	
	timerstring = timermins .. timercolon .. timersecs
	
	if timer <= 10 then
		fontstring = "WTMRR"
	elseif timer <= 30 then
		fontstring = "WTMRY"
	end
	
	local x = xmid
	x = $ - (4 * 9)
	drawFontString(v, fontstring, timerstring, x, y, 18, flags)
	
	local clockpatch
	local clockframe = (p.realtime % 18) / 3
	
	clockpatch = "WTMRCK" .. tostring(clockframe)
	v.draw(x, y, v.cachePatch(clockpatch), flags)
end

local function timeextended(p)
	local rand = P_RandomRange(1, #timeextension)
	local quirp = timeextension[rand]
	S_StartSound(nil,quirp[1],p)
	return quirp[2]
end

local function drawlaps(v, p, map, flags, flash)
	if not (p and p.valid) then return end
	local currentlap
	local maxlaps
	local font = "TMER"
	if tonumber(mapheaderinfo[map].lapcheckpoints) ~= nil then
		currentlap = p.arcade.numlaps + 1
		if p.laps == numlaps then currentlap = p.arcade.numlaps end
		if (p.laps + tonumber(mapheaderinfo[map].lapcheckpoints)) >= numlaps then font = "TMERY" end
		maxlaps = numlaps / tonumber(mapheaderinfo[map].lapcheckpoints)
	else
		currentlap = p.laps + 1
		if p.laps == numlaps then currentlap = p.laps end
		if (p.laps + 1) >= numlaps then font = "TMERY" end
		maxlaps = numlaps 
	end
	
	if flash then font = "TMERG" end
	if daytonakart.hud == 2 then
		font = "WTMR"
		if maxlaps > 9 then
			if currentlap <= 9 then currentlap = "0" .. $ end
			v.drawString(278, 4, "LAP", flags, "right")
			drawFontString(v, font, tostring(currentlap), 280, 4, 18, flags)
		else
			v.drawString(296, 4, "LAP", flags, "right")
			drawFontString(v, font, tostring(currentlap), 298, 4, 18, flags)
		end
		v.drawString(316, 36, "OF "..tostring(maxlaps), flags, "right")
	else
		v.drawString(289, 4, "LAP", flags, "right")
		if maxlaps > 9 then
			if currentlap <= 9 then currentlap = "0" .. $ end
			drawFontString(v, font, tostring(currentlap), 226, 12, 18, flags)
			v.drawString(280, 22, "OF", flags, "right")
			drawFontString(v, font, tostring(maxlaps), 280, 12, 18, flags)
		else
			drawFontString(v, font, tostring(currentlap), 262, 12, 18, flags)
			v.drawString(298, 22, "OF", flags, "right")
			drawFontString(v, font, tostring(maxlaps), 298, 12, 18, flags)
		end
	end
end

local function convertspeed(speed, speedtype, mobjscale)
	local convertedspeed
	local fractionspeed
	local typetext
	local result
	if speedtype == MILESPERHOUR then
		typetext = "MPH"
		convertedspeed = FixedDiv(FixedMul(speed, 88465), mobjscale)/(FRACUNIT/10)
	elseif speedtype == KILOMETERSPERHOUR then
		typetext = "KM/H"
		convertedspeed = FixedDiv(FixedMul(speed, 142371), mobjscale)/(FRACUNIT/10)
	elseif speedtype == METERSPERSECOND then
		typetext = "M/S"
		convertedspeed = FixedDiv(FixedMul(speed, 39547), mobjscale)/(FRACUNIT/10)
	elseif speedtype == FRACUNITSPERTIC then
		typetext = "FU/T"
		convertedspeed = FixedDiv(speed, mobjscale)/(FRACUNIT/10)
	elseif speedtype == NAUTICALMILESPERHOUR then
		typetext = "KNOT"
		convertedspeed = FixedDiv(FixedMul(speed, 76874), mobjscale)/(FRACUNIT/10)
	elseif speedtype == FOOTPERSECOND then
		typetext = "FT/S"
		convertedspeed = FixedDiv(FixedMul(speed, 129747), mobjscale)/(FRACUNIT/10)
	end
	fractionspeed = convertedspeed % 10
	convertedspeed = $ / 10
	return convertedspeed, fractionspeed, typetext
end

local RPMTIC = 125

local function boostchargerpm(charge, perfectRPM, overRPM)
	//Returns 1000 if charge == 0, approx perfectRPM if charge is 35 or 36,
	//slightly over if 37-40, and at least overRPM if 50 or more
	
	local MINRPM = 1000
	local phaseOne = (perfectRPM - MINRPM)/35
	local phaseTwo = perfectRPM/80
	local phaseThree = (overRPM - (perfectRPM + (perfectRPM/20)))/10
	local phaseMax = overRPM/200
	
	local result = MINRPM
	if charge <=35 then
		result = $ + phaseOne*charge
		return result
	end
	result = perfectRPM
	if charge == 36 then return result end
	if charge <= 40 then
		result = $ + phaseTwo*(charge-36)
	elseif charge <= 49 then
		result = $ + phaseTwo*4 + phaseThree*(charge-40)
	else
		result = overRPM + phaseMax*(charge-50)
	end
	return result
end

//This is used to make the custom speedometer
local function drawspeed(v, p, x, y, flags)
	if not (p and p.valid) then return end
	local rpm = 1000 + FixedDiv(FixedMul(FixedDiv(p.speed, mapheaderinfo[gamemap].mobj_scale), (baseRPM[p.kartweight] - 1000)), basemaxspeed[p.kartspeed])
	//Do custom RPM checks for during the start of a race here
	if leveltime <= TIMEFORGO then
		rpm = boostchargerpm(p.kartstuff[k_boostcharge], 8000, 10000)
	end
	//x = 18
	//y = 150
	
	local rpmtics = (rpm/RPMTIC) + 1
	local wholespeed
	local tenthspeed
	local typetext
	local speedtext
	wholespeed, tenthspeed, typetext = convertspeed(p.speed, customspeedometer.value, mapheaderinfo[gamemap].mobj_scale)
	
	//Draw base image
	v.draw(x, y, v.cachePatch("HUDSPEED"), flags)
	
	if rpmtics > 7 then
		if rpmtics < 32 then
			v.drawFill(x+10, y+10, (rpmtics - 8), 5, 187)
			local transparency = (rpm%RPMTIC)/(RPMTIC/10)
			v.draw(x + 2 + rpmtics, y+10, v.cachePatch("HUDSPD1"), flash[transparency])
		else
			v.drawFill(x+10, y+10, 24, 5, 187)
			if rpmtics < 64 then
				v.drawFill(x+34, y+10, (rpmtics - 32), 7, 106)
				local transparency = (rpm%RPMTIC)/(RPMTIC/10)
				v.draw(x + 2 + rpmtics, y+10, v.cachePatch("HUDSPD2"), flash[transparency])
			else
				v.drawFill(x+34, y+10, 32, 7, 106)
				if rpmtics > 87 then
					v.drawFill(x+66, y+10, 24, 11, 133)
				else
					v.drawFill(x+66, y+10, (rpmtics - 64), 11, 133)
					local transparency = (rpm%RPMTIC)/(RPMTIC/10)
					v.draw(x + 2 + rpmtics, y+10, v.cachePatch("HUDSPD3"), flash[transparency])
				end
			end
		end
	end
	
	//Draw Speed
	if tenthspeedometer.value == 0 then
		if tenthspeed > 4 then wholespeed = $ + 1 end
		speedtext = tostring(wholespeed)
	else
		speedtext = wholespeed .. "." .. tenthspeed
	end
	local speedx = 62 + x - (string.len(speedtext) * 12)
	drawMKtext(v, speedtext, speedx, y+22, flags)
	v.drawString(x+64, y+28, typetext, flags, "left")
end

local function drawBattleManiaSpeedTimer(v, p, timertime, flags)
	if not (p and p.valid) then return end
	local rpm = 1000 + FixedDiv(FixedMul(FixedDiv(p.speed, mapheaderinfo[gamemap].mobj_scale), (baseRPM[p.kartweight] - 1000)), basemaxspeed[p.kartspeed])
	//Do custom RPM checks for during the start of a race here
	if leveltime <= TIMEFORGO then
		rpm = boostchargerpm(p.kartstuff[k_boostcharge], 8000, 10000)
	end
	
	local x = 160
	local y = 5
	
	local rpmstage = min((rpm/1000), 11)
	local wholespeed
	local tenthspeed
	local typetext
	local speedtext
	local speedx = 200
	wholespeed, tenthspeed, typetext = convertspeed(p.speed, customspeedometer.value, mapheaderinfo[gamemap].mobj_scale)
	if tenthspeedometer.value == 0 then
		if tenthspeed > 4 then wholespeed = $ + 1 end
		speedtext = tostring(wholespeed) .. " " .. typetext
	else
		speedtext = wholespeed .. "." .. tenthspeed .. " " .. typetext
		speedx = $ + 12
	end
	if string.len(typetext) == 3 then speedx = $ - 8 end
	v.draw(x, y, v.cachePatch("S_REV_"..rpmstage), flags, ((rpmstage > 8) and v.getColormap(0, SKINCOLOR_RED)) or (((rpmstage > 5) and (rpmstage <= 8)) and v.getColormap(0, SKINCOLOR_YELLOW)) or (((rpmstage > 1) and (rpmstage <=5)) and v.getColormap(0, SKINCOLOR_EMERALD)) or v.getColormap(0, SKINCOLOR_COBALT))
	local transparency = (rpm%1000)/100
	if rpmstage < 11 then
		v.draw(x, y, v.cachePatch("S_MID_"..(rpmstage + 1)), flash[transparency]|V_SNAPTOTOP, ((rpmstage > 8) and v.getColormap(0, SKINCOLOR_RED)) or (((rpmstage > 5) and (rpmstage <= 8)) and v.getColormap(0, SKINCOLOR_YELLOW)) or (((rpmstage > 1) and (rpmstage <=5)) and v.getColormap(0, SKINCOLOR_EMERALD)) or v.getColormap(0, SKINCOLOR_COBALT))
	end
	v.drawString(speedx, y+36, speedtext, flags, "right")
	
	if p.arcade.extendflash > 0 and (p.arcade.extendflash % 10) < 5 then
		drawManiaTime(v, timertime, x+1, y+16, flags, GREENTEXT)
	elseif (timertime < 10) and ((p.realtime % 10) < 5) then
		drawManiaTime(v, timertime, x+1, y+16, flags, REDTEXT)
	else
		drawManiaTime(v, timertime, x+1, y+16, flags)
	end
end

local function drawWarioLandSpeed(v, p, flags)
	if not (p and p.valid) then return end
	local wholespeed
	local tenthspeed
	local typetext
	local speedtext
	local speedx
	wholespeed, tenthspeed, typetext = convertspeed(p.speed, customspeedometer.value, mapheaderinfo[gamemap].mobj_scale)
	if tenthspeedometer.value == 0 then
		if tenthspeed > 4 then wholespeed = $ + 1 end
		speedx = 50 - (string.len(tostring(wholespeed)) * 9)
		drawSmallWarioLandNumber(v, wholespeed, speedx, 172, flags)
	else
		speedx = 32 - (string.len(tostring(wholespeed)) * 9)
		drawSmallWarioLandNumber(v, wholespeed, speedx, 172, flags)
		drawLetterScaled(v, "WTMR", ".", 32*FRACUNIT, 172*FRACUNIT, FRACUNIT/2, flags)
		drawLetterScaled(v, "WTMR", tenthspeed, 41*FRACUNIT, 172*FRACUNIT, FRACUNIT/2, flags)
	end
	v.drawString(52, 178, typetext, flags, "left")
end

local function drawWarioLandHP(v, p, flags)
	if not (p and p.valid) then return end
	local barxpos = 1
	local textxpos = 56
	local barypos = 160
	local textypos = 159
	
	if not (hpmod and hpmod.maxhp and p.hpmod) then return end
	local health = p.hpmod.hp
	local hpstep = FixedInt(FixedDiv(health*40, hpmod.maxhp))
	local hpstepcarry = ((health*40)%hpmod.maxhp > 0) and 1 or 0
	if hpstep >= 4 then 
		if hpstep%4 == 0 then
			if hpstepcarry then hpstepcarry = $ + 1 end
			hpstep = $ + ((hpstep-1)/4)
		else
			hpstep = $ + (hpstep/4)
		end
	end
	dkartdebug.hp = hpstep
	local regenhealth = p.hpmod.minhp
	local rhpstep = FixedInt(FixedDiv(regenhealth*40, hpmod.maxhp))
	if rhpstep >= 4 then rhpstep = $ + ((rhpstep-1) / 4) end
	dkartdebug.regen = rhpstep
	local latenthealth = p.hpmod.laghp
	local lhpstep = FixedInt(FixedDiv(latenthealth*40, hpmod.maxhp))
	if lhpstep >= 4 then lhpstep = $ + ((lhpstep-1) / 4) end
	dkartdebug.latent = lhpstep
	
	local temphpstep = hpstep
	local xoff = 0
	local s = 1
	local i = 1
	while temphpstep ~= 0 do
		local bit = temphpstep&s
		temphpstep = temphpstep&!s
		if bit then
			v.draw(barxpos + xoff, barypos, v.cachePatch("WLKHPU"..i), flags, v.getColormap(0, SKINCOLOR_GREEN))
			xoff = $+s
		end
		i = $+1
		s = $<<1
	end
	
	if lhpstep > hpstep then
		hpstepcarry = 0
		temphpstep = lhpstep - hpstep
		if temphpstep > (49 - xoff) then temphpstep = max(49 - xoff, 0) end
		s = 1
		i = 1
		while temphpstep ~= 0 do
			local bit = temphpstep&s
			temphpstep = temphpstep&!s
			if bit then
				v.draw(barxpos + xoff, barypos, v.cachePatch("WLKHPU"..i), flags, v.getColormap(0, SKINCOLOR_CRIMSON))
				xoff = $+s
			end
			i = $+1
			s = $<<1
		end
	end
	
	if rhpstep > hpstep then
		hpstepcarry = 0
		temphpstep = rhpstep - hpstep
		if temphpstep > (49 - xoff) then temphpstep = max(49 - xoff, 0) end
		s = 1
		i = 1
		while temphpstep ~= 0 do
			local bit = temphpstep&s
			temphpstep = temphpstep&!s
			if bit then
				v.draw(barxpos + xoff, barypos, v.cachePatch("WLKHPU"..i), flags, v.getColormap(0, SKINCOLOR_SAPPHIRE))
				xoff = $+s
			end
			i = $+1
			s = $<<1
		end
	end
	
	if hpstepcarry > 0 then
		v.draw(barxpos + xoff + (hpstepcarry - 1), barypos, v.cachePatch("WLKHPU1"), flags, v.getColormap(0, SKINCOLOR_YELLOW))
		xoff = $ + hpstepcarry
	end
	
	temphpstep = max(49 - xoff, 0)
	s = 1
	i = 1
	while temphpstep ~= 0 do
		local bit = temphpstep&s
		temphpstep = temphpstep&!s
		if bit then
			v.draw(barxpos + xoff, barypos, v.cachePatch("WLKHPU"..i), flags, v.getColormap(0, SKINCOLOR_BLACK))
			xoff = $+s
		end
		i = $+1
		s = $<<1
	end
	
	v.draw(barxpos, barypos, v.cachePatch("WLKHPB"), flags)
	
	v.drawString(textxpos, textypos, health .. "HP", flags|((hpstep<4 and V_REDMAP) or (hpstep<10 and V_YELLOWMAP) or V_GREENMAP))
end

local function drawplace(v, p, numplayers, flags, color)
	if not (p and p.valid) then return end
	local pxpos = 270
	local txpos = 300
	local ypos = 170
	if daytonakart.hud == 3 then
		txpos = 240
		pxpos = 210
	end
	local placetext = tostring(p.kartstuff[k_position])
	local placefont = "TMER"
	if daytonakart.hud == 2 then placefont = "WTMR" end
	if color == REDTEXT then placefont = $ .. "R" end
	if color == GREENTEXT then placefont = $ .. "G" end
	if color == YELLOWTEXT then placefont = $ .. "Y" end
	if numplayers == 1 then //Single Player
		hud.disable("freeplay")
	end
	if daytonakart.hud == 2 then
		pxpos = 280
		ypos = $ - 4
		local positiontext = "TH"
		if p.kartstuff[k_position] == 1 then positiontext = "ST" end
		if p.kartstuff[k_position] == 2 then positiontext = "ND" end
		if p.kartstuff[k_position] == 3 then positiontext = "RD" end
		if numplayers > 9 then
			if p.kartstuff[k_position] < 10 then 
				if leadingzero.value == 1 then
					placetext = "0" .. $
					pxpos = $ - 18
				end
			end
		end
		v.drawString(txpos, ypos, positiontext, flags) 
		drawFontString(v, placefont, placetext, pxpos, ypos, 18, flags)
	else
		if numplayers > 9 then //10+ Players
			pxpos = $ - 36
			txpos = $ - 18
			if p.kartstuff[k_position] < 10 then 
				if leadingzero.value == 0 then
					pxpos = $ + 9
				else
					placetext = "0" .. $
				end
			end
		end
		v.drawString(txpos - 6 , ypos - 8, "PLACE", flags, "center")
		drawFontString(v, placefont, placetext, pxpos, ypos, 18, flags)
		drawMKtext(v, "/", txpos - 12, ypos + 7, flags)
		drawFontString(v, "TMER", tostring(numplayers), txpos, ypos, 18, flags)
	end
end

addHook("NetVars", function(network)
	endtimer = network(endtimer)
	daytonakart.active = network(daytonakart.active)
end)

addHook("MapChange", do
	endtimer = 0
	daytonakart.active = arcadeon.value
end)

addHook("MapLoad", do
	if not (gametype & GT_MATCH) then
		daytonakart.active = arcadeon.value
		if mapheaderinfo[gamemap].forcedefaulthud then
			daytonakart.hud = 1
		elseif mapheaderinfo[gamemap].forcewariohud then
			daytonakart.hud = WARIOLANDHUD
		elseif mapheaderinfo[gamemap].forcemaniahud then
			daytonakart.hud = BATTLEMANIAHUD
		elseif mapheaderinfo[gamemap].forcehud then
			local newhud = tonumber(mapheaderinfo[gamemap].forcehud)
			if (newhud < 1) or (newhud > 3) then newhud = hudtype.value end
			daytonakart.hud = newhud
		else
			daytonakart.hud = hudtype.value
		end
		daytonakart.timeover = hpmodtimeover.value
		daytonakart.diff = difficulty.value
		if timeoverdiffflag.value then
			daytonakart.topercent = 1
			daytonakart.torate = timeoverdiffdefault[daytonakart.diff]
			daytonakart.todam = 1
		else
			daytonakart.topercent = timeoverpercent.value
			daytonakart.torate = timeoverrate.value
			daytonakart.todam = timeoverdam.value
		end
		if modeattacking then 
			daytonakart.hud = TAHUD
			daytonakart.diff = 0
			daytonakart.timeover = 0
			daytonakart.active = 1
			for m in thinkers.iterate("mobj")
				if m and m.valid and m.type == MT_RANDOMITEM
					m.state = S_INVISIBLE
					m.tics = 1
					m.flags = MF_NOBLOCKMAP
				end
			end
		end
		if daytonakart.active then
			COM_BufInsertText(server, "karteliminatelast off")
			COM_BufInsertText(server, "countdowntime 600")
			for p in players.iterate do
				startset(p, gamemap)
			end
		end
	end
end)

local numplayers

addHook("ThinkFrame", do
	if (not (gametype & GT_MATCH)) and daytonakart.active then
		local outplayers
		numplayers = 0
		outplayers = 0
		for p in players.iterate do
			if not p.arcade then startset(p, gamemap) end
			p.kartstuff[k_lapanimation] = 0 //Thanks Lat' for showing me that this is possible.
			if leveltime < 26 then
				continue --Don't do anything during the first few frames as we need to set things up
			elseif leveltime == 26 then
				startset(p, gamemap) //Just in case
			elseif leveltime == TIMEFORTHREE then
				S_StartSound(nil,sfx_amsc3,p)
			elseif leveltime == TIMEFORTWO then
				S_StartSound(nil,sfx_amsc2,p)
			elseif leveltime == TIMEFORONE then
				S_StartSound(nil,sfx_amsc1,p)
			elseif leveltime == TIMEFORGO then
				S_StartSound(nil,sfx_amscgo,p)
				if hpmod and hpmod.running and hpmod.maxhp ~= p.arcade.hpcap then
					p.hpmod = {
						hp = hpmod.maxhp,
						minhp = hpmod.maxhp,
						laghp = hpmod.maxhp,
						deathtimer = 0,
						lastlap = 0
					}
					p.arcade.hpcap = hpmod.maxhp
					player.arcade.hpcapped = true
				end
			elseif leveltime == TIMEFORGO + TICRATE then
				S_StartSound(nil,sfx_amatuw,p)
				if hpmod and hpmod.running and not player.arcade.hpcapped then
					p.hpmod = {
						hp = hpmod.maxhp,
						minhp = hpmod.maxhp,
						laghp = hpmod.maxhp,
						deathtimer = 0,
						lastlap = 0
					}
					p.arcade.hpcap = hpmod.maxhp
					player.arcade.hpcapped = true
				end
			elseif leveltime > TIMEFORGO and hpmod and hpmod.running and not player.arcade.hpcapped then
				p.hpmod = {
					hp = hpmod.maxhp,
					minhp = hpmod.maxhp,
					laghp = hpmod.maxhp,
					deathtimer = 0,
					lastlap = 0
				}
				p.arcade.hpcap = hpmod.maxhp
				player.arcade.hpcapped = true
			end
			if p.spectator then continue end
			if p.arcade == nil then startset(p, gamemap) end
			numplayers = $ + 1
			
			if p.exiting then
				outplayers = $ + 1 
				if p.arcade.quirplength == 0 then
					p.arcade.quirplength = -1
					S_StartSound(nil,sfx_amgoal,p)
				end
				continue
			end
			
			if hpmod and hpmod.running and (p.pflags & PF_TIMEOVER) then 
				if p.arcade.extendflash > -TIMEOVERLENGTH then p.arcade.extendflash = $ - 1 end
				outplayers = $ + 1
				continue
			end
			
			if sneakeronly.value == 1 and (p.kartstuff[k_itemtype] > 3) and (p.kartstuff[k_itemtype] < 13) then
				local olditem = p.kartstuff[k_itemtype]
				local newitem = ITEMCHANGE[olditem]
				if newitem > 0 then p.kartstuff[k_itemtype] = newitem end
				if p.kartstuff[k_itemamount] >= 4 then //10 Bananas or 4x Orbinauts
					p.kartstuff[k_itemamount] = 3
				elseif p.kartstuff[k_itemamount] == 3 then //Triple Orbinauts/Bananas
					p.kartstuff[k_itemamount] = 2
				elseif p.kartstuff[k_itemamount] == 2 and olditem == 7 then //Twin Jaws
					p.kartstuff[k_itemamount] = 3
				elseif olditem == 9 or olditem == 7 then //Ballhogs or Single Jawz
					p.kartstuff[k_itemamount] = 2
				end
			end
			
			if p.arcade.timelimit == p.realtime and not p.arcade.timeover then //Time Over
				p.arcade.timeover = true
				S_StartSound(nil,sfx_arcko,p)
				if hitfeed then
					local patch = "HFEDCK"
					local source = {{nil, nil}, {nil, nil}, nil}
					local victim = {{p.name, p.mo.hf_overridenamecolor or p.skincolor}, {skins[p.mo.skin].facemmap, p.skincolor}, p}
					if not HF_animIcon[patch] then HF_animIcon[patch] = {6, 3} end
					HF_SendHitMessage(source, victim, patch)
				end
			elseif p.arcade.extendflash < 0 and not (p.arcade.extendflash == -TIMEOVERLENGTH) then
				p.arcade.extendflash = $ - 1
			end
			
			if p.arcade.timeover then
				if hpmod and hpmod.running and daytonakart.timeover then
					if not (p.kartstuff[k_driftboost] or (p.kartstuff[k_growshrinktimer] > 0) or p.kartstuff[k_sneakertimer]) then
						p.kartstuff[k_accelboost] = -(FRACUNIT / 5) //Your engine becomes less effecient if time is over
					end
					if (p.realtime - p.arcade.timelimit)%daytonakart.torate == 0 then --Time to take some damage!
						local dam
						if daytonakart.topercent then
							dam = max((hpmod.maxhp * daytonakart.todam) / 100, 1) --Always do at least 1HP damage
						else
							dam = daytonakart.todam
						end
						if dam > p.hpmod.hp then dam = p.hpmod.hp end
						if rawget(_G, "hpmod_damagehp") then --This allows this to do the damage properly
							hpmod_damagehp(p, dam, {{"Time Itself", 0}, {"HFEDCK6", 0}})
						else --No? Well, this is a 'hack' to do it instead until it's added in
							p.hpmod.hp = $-dam 
						end
						p.arcade.hpcap = $-dam //This damage also damages the HP Cap as well for the player.
						if p.arcade.hpcap < p.hpmod.minhp then p.hpmod.minhp = p.arcade.hpcap end
					end
				else
					p.pflags = $ | PF_STASIS
					if p.speed == 0 then
						outplayers = $ + 1
						if p.arcade.extendflash > -1 then
							p.arcade.extendflash = -1
							S_ChangeMusic("KRFAIL",true,p)
							if hitfeed then
								local patch = "HFEDCK"
								local source = {{"Time Itself", 0}, {nil, nil}, nil}
								local victim = {{p.name, SKINCOLOR_CRIMSON}, {skins[p.mo.skin].facemmap, p.skincolor}, p}
								if not HF_animIcon[patch] then HF_animIcon[patch] = {6, 5} end
								HF_SendHitMessage(source, victim, patch)
							end
						end
						continue
					end
				end
			else
				if p.hpmod and p.arcade.hpcap then
					if p.arcade.hpcap < p.hpmod.hp then p.hpmod.hp = p.arcade.hpcap end
					if p.arcade.hpcap < p.hpmod.minhp then p.hpmod.minhp = p.arcade.hpcap end
				end
			end
			
			if (p.laps > p.arcade.lapcheck) and p.arcade.extendflash > -1 then
				p.arcade.lapcheck = $ + 1
				p.arcade.timetoshow = (p.arcade.timelimit - p.realtime) / TICRATE
				if mapheaderinfo[gamemap].noextension then
					p.arcade.numlaps = $ + 1
					p.arcade.quirplength = TICRATE
					if p.arcade.numlaps == 1 then p.arcade.extendflash = FLASHTIME end
				elseif tsrb2kr then
					local team = server.tsr_teams[p.tsr_team]
					local doteamtimebonus = true
					local toplap = 0
					for i = 1, #team do
						local teammate = team[i]
						toplap = max($, teammate.laps)
						if teammate == p then continue end
						if teammate.arcade.lapcheck >= p.laps then doteamtimebonus = false end
					end
					if doteamtimebonus then
						for i = 1, #team do
							local teammate = team[i]
							if teammate.arcade.timeover then //Almost about to lose, but crosses the finish line before stopping
								teammate.arcade.timeover = false //Back in the game!
								teammate.arcade.timeincrease = 1
								teammate.arcade.timelimit = (1 + (teammate.realtime / TICRATE)) * TICRATE
							end
							if not (p.arcade.lapcheck == numlaps) then
								teammate.arcade.timelimit = $ + teammate.arcade.bonustime
								teammate.arcade.timeincrease = $ + (teammate.arcade.bonustime / TICRATE)
								teammate.arcade.extendflash = FLASHTIME
							end
							if (tonumber(mapheaderinfo[gamemap].lapcheckpoints) ~= nil) and (p == teammate) then
								if (p.arcade.lapcheck % tonumber(mapheaderinfo[gamemap].lapcheckpoints)) == 0 then
									p.arcade.numlaps = $ + 1
									p.arcade.quirplength = timeextended(p)
								end
							else
								if p ~= teammate then teammate.arcade.teamtimebonus = true end
								p.arcade.quirplength = timeextended(p)
							end
						end
						TSR_fillUltimate(p, 120, "TIME EXTEND!", sfx_arcbt, true)
					else
						p.arcade.quirplength = max(35, $)
						p.arcade.teamtimebonus = false
					end
				else
					if p.arcade.timeover then //Almost about to lose, but crosses the finish line before stopping
						p.arcade.timeover = false //Back in the game!
						p.arcade.timeincrease = 1
						p.arcade.timelimit = (1 + (p.realtime / TICRATE)) * TICRATE
					end
					if not (p.arcade.lapcheck == numlaps) then
						p.arcade.timelimit = $ + p.arcade.bonustime
						p.arcade.timeincrease = $ + (p.arcade.bonustime / TICRATE)
						p.arcade.extendflash = FLASHTIME
					end
					if tonumber(mapheaderinfo[gamemap].lapcheckpoints) ~= nil then
						if (p.arcade.lapcheck % tonumber(mapheaderinfo[gamemap].lapcheckpoints)) == 0 then
							p.arcade.numlaps = $ + 1
							p.arcade.quirplength = timeextended(p)
						end
					else
						p.arcade.quirplength = timeextended(p)
					end
				end
			elseif (p.arcade.extendflash > 0) then //Reduce countdown on Lap Bonus
				p.arcade.extendflash = $ - 1
			end
			if (p.arcade.timeincrease > 0) and (p.realtime % UPDATESPEED) == 0 then
				p.arcade.timetoshow = $ + 1
				p.arcade.timeincrease = $ - 1
			end
			if (p.arcade.timeincrease > 0) and (p.realtime % 35) == 0 then
				p.arcade.timetoshow = $ - 1
			end
			
			p.arcade.wariotime = p.arcade.timetoshow
			if (((p.arcade.timelimit - p.realtime) % TICRATE) > 0) and (leveltime > TIMEFORGO) then p.arcade.wariotime = $ + 1 end
			
			if p.arcade.quirplength > 0 then
				p.arcade.quirplength = $ - 1
				if (p.arcade.quirplength == 0) and not p.arcade.teamtimebonus then
					local lapstogo 
					if tonumber(mapheaderinfo[gamemap].lapcheckpoints) ~= nil then
						lapstogo = (numlaps/tonumber(mapheaderinfo[gamemap].lapcheckpoints)) - p.arcade.numlaps
					else
						lapstogo = numlaps - p.arcade.lapcheck
					end
					if lapstogo == 1 then S_StartSound(nil,sfx_am1lpr,p) end
					if lapstogo == 2 then S_StartSound(nil,sfx_am2lpr,p) end
					if lapstogo == 5 then S_StartSound(nil,sfx_am5lpr,p) end
					if lapstogo == 7 then S_StartSound(nil,sfx_am7lpr,p) end
				end
			end
			
			if (p.arcade.timelimit - (30 * TICRATE)) == p.realtime and p.realtime > 0 then
				//Keep an eye on your time!
				S_StartSound(nil,sfx_amlot1,p)
			elseif (p.arcade.timelimit - (13 * TICRATE)) == p.realtime and p.realtime > 0  then
				//Almot out of time!
				S_StartSound(nil,sfx_amlot2,p)
			elseif (p.arcade.timelimit - (10 * TICRATE)) == p.realtime and p.realtime > 0  then
				//Play a warning SFX to give an audio cue that time is about to run out.
				S_StartSound(nil,sfx_arccr,p)
			elseif (p.arcade.timelimit - (10 * TICRATE)) < p.realtime and p.realtime > 0 
				and (p.realtime % (TICRATE)) == 0
				and not (p.arcade.timelimit <= p.realtime) then
				//Time's running out!
				S_StartSound(nil,sfx_arclo,p)
			end
		end
		
		if endtimer > 0 then
			endtimer = $ + 1
			if endtimer > TIMEOVERLENGTH and not modeattacking then G_ExitLevel() end
		elseif outplayers == numplayers and numplayers > 0 then
			endtimer = 1
		end
	else
		for p in players.iterate do //Still do the countdown from the announcer
			if leveltime == TIMEFORTHREE then
				S_StartSound(nil,sfx_amsc3,p)
			elseif leveltime == TIMEFORTWO then
				S_StartSound(nil,sfx_amsc2,p)
			elseif leveltime == TIMEFORONE then
				S_StartSound(nil,sfx_amsc1,p)
			elseif leveltime == TIMEFORGO then
				S_StartSound(nil,sfx_amscgo,p)
			elseif leveltime == TIMEFORGO + TICRATE then
				S_StartSound(nil,sfx_amatuw,p)
			end
		end
	end
end)

hud.add(function(v, p)
	if not (gametype & GT_MATCH) and (not p.spectator) and (p.arcade != nil) and (not splitscreen) and daytonakart.active then --edited by LunarRay to disable HUD in splitscreen
		local timertime
		hud.disable("time") //Done custom
		hud.disable("gametypeinfo") //Also done custom
		hud.disable("speedometer") //Also done custom
		hud.disable("position") //Also done custom
		if server.tsr_finishtimer and server.tsr_finishtimer > 0 then return end
		//We don't need this while the results screen is being done
		if (p.arcade.timeincrease > 0) and not (instaupdate.value == 1) then
			timertime = p.arcade.timetoshow
			if daytonakart.hud == WARIOLANDHUD then timertime = p.arcade.wariotime end
		elseif (leveltime < TIMEFORONE) and not (instaupdate.value == 1) then
			timertime = (p.arcade.timelimit/35) - (max(((TIMEFORONE - leveltime)/(UPDATESPEED*2)), 0) * 5)
		else
			timertime = (p.arcade.timelimit - p.realtime) / 35
			if daytonakart.hud == WARIOLANDHUD and ((((p.arcade.timelimit - p.realtime) / 35) > 0) or ((p.arcade.timelimit > p.realtime) and (p.arcade.timelimit < (p.realtime + 35)))) and (leveltime > TIMEFORGO) then timertime = $ + 1 end
		end
		local moretime = false
		if timertime <= 0 then timertime = 0 end
		if (daytonakart.hud ~= WARIOLANDHUD) then
			if timertime > 99 and (timercap.value == 1) then //Capped at 99s
				timertime = 99
				moretime = true
			elseif timertime > 150 and (timercap.value == 2) then //Capped at 150s like in Inital D Arcade Stages
				timertime = 150
			end
		end
		if (p.arcade.timeover and p.arcade.extendflash < 0)
			or (hpmod and hpmod.running and (p.pflags & PF_TIMEOVER)) then
			hud.disable("minimap")
			hud.disable("item")
			hud.disable("rankings")
			if minrankplus then
				if minrankplus.enabled then minrankplus.enabled = false end
			else
				hud.disable("minirankings")
			end
			
			local tic = abs(p.arcade.extendflash)
			
			//TIME
			if daytonakart.hud == WARIOLANDHUD then
				drawWarioLandHP(v, p, V_SNAPTOLEFT|
				flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
				drawWarioLandTimer(v, p, timertime, 160, 2, flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
			elseif daytonakart.hud == BATTLEMANIAHUD then
				drawBattleManiaSpeedTimer(v, p, timertime, V_SNAPTOTOP|flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
			elseif daytonakart.hud == TAHUD then
				drawTATimer(v, p, timertime, 160, 10, V_SNAPTOTOP|flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
				drawTAMedalTimes(v, p, 4, 4, V_SNAPTOTOP|V_SNAPTOLEFT|flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
			else
				v.drawString(160, 2, "TIME", V_SNAPTOTOP|flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH], "center")
				drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH], REDTEXT)
			end
			if (customspeedometer.value > 0) and (not tsrb2kr) then 
				if daytonakart.hud == WARIOLANDHUD then
					drawWarioLandSpeed(v, p, flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
				elseif daytonakart.hud ~= BATTLEMANIAHUD then
					drawspeed(v, p, 4, 150, flash[(TIMEOVERLENGTH-tic)*10/TIMEOVERLENGTH])
				end
			end
			
			if not (modeattacking or tsrb2kr) then
				drawplace(v, p, numplayers, V_HUDTRANS, REDTEXT) //Still shows the final position
			end
			
			//TIME OVER
			if tic < (TIMEOVERLENGTH / 2) then
				v.draw(0, -60, v.cachePatch("BLACKSQR"), flash[(tic*16)/TIMEOVERLENGTH])
			else
				v.draw(0, -60, v.cachePatch("BLACKSQR"), V_20TRANS)
			end
			if not (hpmod and hpmod.running and (p.pflags & PF_TIMEOVER)) then
				if tic >= (TIMEOVERLENGTH / 4) and tic < ((3 * TIMEOVERLENGTH) / 4) then
					v.draw(160, 100, v.cachePatch("TIMEOVER"), flash[((tic - (TIMEOVERLENGTH / 4))*20)/TIMEOVERLENGTH])
				elseif tic >= ((3 * TIMEOVERLENGTH) / 4) then
					v.draw(160, 100, v.cachePatch("TIMEOVER"))
				end
			end
			if modeattacking then
				if endtimer >= (TIMEOVERLENGTH / 4) and endtimer < ((3 * TIMEOVERLENGTH) / 4) then
					v.drawString(160, 92, "RUN OVER - DNF", flash[((frame - (TIMEOVERLENGTH / 4))*20)/TIMEOVERLENGTH], "center")
					v.drawString(160, 100, "PLEASE RETRY OR ABORT TO CONTINUE", flash[((frame - (TIMEOVERLENGTH / 4))*20)/TIMEOVERLENGTH], "center")
				elseif endtimer >= ((3 * TIMEOVERLENGTH) / 4) then
					v.drawString(160, 92, "RUN OVER - DNF", 0, "center")
					v.drawString(160, 100, "PLEASE RETRY OR ABORT TO CONTINUE", 0, "center")
				end
			end
			
		else
			hud.enable("minimap")
			hud.enable("item")
			hud.enable("rankings")

			if not (minrankplus or tsrb2kr) then
				hud.enable("minirankings")
			elseif minrankplus and not (minrankplus.enabled or tsrb2kr) then
				minrankplus.enabled = true
			end
			
			//TIME
			if (daytonakart.hud ~= TAHUD) and (daytonakart.hud ~= WARIOLANDHUD) and (daytonakart.hud ~= BATTLEMANIAHUD) then
				v.drawString(160, 2, "TIME", V_HUDTRANS, "center")
			end
			if moretime == true then
				if (daytonakart.hud ~= BATTLEMANIAHUD) then
					v.drawString(178, 12, "+", V_HUDTRANS, "center")
				else
					v.drawString(179, 22, "+", V_HUDTRANS|(((p.arcade.extendflash > 0) and (p.arcade.extendflash % 10) < 5) and V_GREENMAP or V_YELLOWMAP), "center")
				end
			end
			if daytonakart.hud == WARIOLANDHUD then
				drawWarioLandHP(v, p, V_SNAPTOLEFT|V_HUDTRANS)
				drawWarioLandTimer(v, p, timertime, 160, 2, V_SNAPTOTOP|V_HUDTRANS)
				if dkartdebug.enabled then
					v.drawString(2, 2,  "  HP: "..dkartdebug.hp)
					v.drawString(2, 12, "LHP: "..dkartdebug.latent)
					v.drawString(2, 22, "RHP: "..dkartdebug.regen)
				end
			elseif daytonakart.hud == BATTLEMANIAHUD then
				drawBattleManiaSpeedTimer(v, p, timertime, V_SNAPTOTOP|V_HUDTRANS)
			elseif daytonakart.hud == TAHUD then
				drawTATimer(v, p, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS)
				drawTAMedalTimes(v, p, 4, 4, V_SNAPTOTOP|V_SNAPTOLEFT|V_HUDTRANS)
			elseif p.exiting then
				if p.kartstuff[k_position] == 1 then
					drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS, YELLOWTEXT) //Yellow
				elseif p.pflags & PF_TIMEOVER then
					drawTimer(v, 0, 160, 10, V_SNAPTOTOP|V_HUDTRANS, REDTEXT) //Red	
				elseif K_IsPlayerLosing(p) then
					drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS) //White
				else
					drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS, GREENTEXT) //Green
				end
			elseif p.arcade.extendflash > 0 and (p.arcade.extendflash % 8) < 4 then
				drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS, GREENTEXT) //Green
			elseif (timertime > 9) or ((p.realtime % 8) < 4) or (leveltime < TIMEFORONE) then
				drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS) //White
			else
				drawTimer(v, timertime, 160, 10, V_SNAPTOTOP|V_HUDTRANS, REDTEXT) //Red	
			end
			if mapheaderinfo[gamemap].noextension then
				if p.arcade.extendflash > 0 and (p.arcade.extendflash % 10) < 5 then
					if daytonakart.hud == WARIOLANDHUD then
						v.drawString(160, 34, "NO TIME EXTENSIONS!", V_SNAPTOTOP|V_HUDTRANS|V_REDMAP, "center")
					elseif daytonakart.hud == BATTLEMANIAHUD then
						v.drawString(160, 53, "NO TIME EXTENSIONS!", V_SNAPTOTOP|V_HUDTRANS|V_REDMAP, "center")
					elseif daytonakart.hud == TAHUD then
						v.drawString(160, 48, "NO TIME EXTENSIONS!", V_SNAPTOTOP|V_HUDTRANS|V_REDMAP, "center")
					else
						v.drawString(160, 38, "NO TIME EXTENSIONS!", V_SNAPTOTOP|V_HUDTRANS|V_REDMAP, "center")
					end
				end
				if not tsrb2kr then
					drawlaps(v, p, gamemap, V_HUDTRANS|V_SNAPTORIGHT, true)
				end
			elseif p.arcade.extendflash > 0 and (p.arcade.extendflash % 10) < 5 and (not tsrb2kr) then
				//Flash Time Extended image
				if daytonakart.hud == WARIOLANDHUD then
					v.drawString(160, 34, "TIME EXTENDED", V_SNAPTOTOP|V_HUDTRANS, "center")
				elseif daytonakart.hud == BATTLEMANIAHUD then
					v.drawString(160, 53, "TIME EXTENDED", V_SNAPTOTOP|V_HUDTRANS, "center")
				elseif daytonakart.hud == TAHUD then
					v.drawString(160, 48, "TIME EXTENDED", V_SNAPTOTOP|V_HUDTRANS, "center")
				else
					v.drawString(160, 38, "TIME EXTENDED", V_SNAPTOTOP|V_HUDTRANS, "center")
				end
				//LAPS
				if not tsrb2kr then
					drawlaps(v, p, gamemap, V_HUDTRANS|V_SNAPTORIGHT, true)
				end
			elseif not tsrb2kr then
				//LAPS
				drawlaps(v, p, gamemap, V_HUDTRANS|V_SNAPTORIGHT)
			end
			
			//Speedometer
			if (customspeedometer.value > 0) and (not tsrb2kr) then 
				if daytonakart.hud == 2 then
					drawWarioLandSpeed(v, p, V_HUDTRANS)
				elseif daytonakart.hud ~= BATTLEMANIAHUD then
					drawspeed(v, p, 4, 150, V_HUDTRANS)
				end
			end
			
			//Placement
			if not (modeattacking or tsrb2kr) then
				local lapstogo = numlaps - p.laps
				if mapheaderinfo[gamemap].lapcheckpoints ~= nil and lapstogo > 0 then
					lapstogo = (numlaps/mapheaderinfo[gamemap].lapcheckpoints) - p.arcade.numlaps
				end
				
				local colour = nil
				
				if lapstogo == 0 then //FINISH!
					if p.kartstuff[k_position] == 1 then
						colour = YELLOWTEXT
					elseif K_IsPlayerLosing(p) then //Red
						colour = REDTEXT
					else //Green
						colour = GREENTEXT
					end
				elseif lapstogo == 1 and (p.realtime % 12) < 6 then //Final lap!
					if K_IsPlayerLosing(p) then //Flash Red
						colour = REDTEXT
					else //Flash Green
						colour = GREENTEXT
					end
				end
				
				if colour ~= nil then
					drawplace(v, p, numplayers, V_HUDTRANS, colour)
				else
					drawplace(v, p, numplayers, V_HUDTRANS)
				end
			end
		end
	else
		if not tsrb2kr then
			hud.enable("time")
			hud.enable("item")
			hud.enable("position")
			hud.enable("speedometer")
		end
		
		hud.enable("minimap")
		
		
		if not (tsrb2kr or (hpmod and hpmod.running and G_BattleGametype())) then
			hud.enable("gametypeinfo")
		end
		
		hud.enable("rankings")
		
		if not (tsrb2kr or minrankplus) then
			hud.enable("minirankings")
		elseif minrankplus and not (minrankplus.enabled or tsrb2kr) then
			minrankplus.enabled = true
		end
		
		hud.enable("freeplay")
	end
end, "game")

hud.add(function(v)
	local difficultytext = ({
		[30] = "N00B Level",
		[15] = "Easy Mode",
		[0] = "Standard",
		[-5] = "Advanced",
		[-10] = "Expert",
		[-20] = "Master"
	}) [daytonakart.diff]
	v.drawString(0, 192, difficultytext, V_ALLOWLOWERCASE|V_40TRANS)
	v.drawString(320, 192, "Daytona Kart: "..version.." ", V_ALLOWLOWERCASE|V_40TRANS, "right")
end, "scores")

print("\x82".."Daytona Kart "..version.." is now installed!")

COM_AddCommand("daytonaHelp", function(p)
	CONS_Printf(p, "\x82".."arcade_speed: Determines what speedometer type to use. Default Miles.")
	CONS_Printf(p, "\x82".."Options - Disabled, Miles, Kilometers, Meters, Fracunits, Knots, Feet.")
	CONS_Printf(p, "\x82".."arcade_tenthspeed: Shows or hides the tenths of the speed. Default On.")
	CONS_Printf(p, "\x82".."arcade_capto99: Caps the shown time limit to 99. Default Off.")
	CONS_Printf(p, "\x82".."arcade_placezero: Shows a leading zero in 10+ player races. Default Off.")
	CONS_Printf(p, "\x82".."arcade_instupdate: Does the timer insta-updates, or over time? Default On.")
	CONS_Printf(p, "\x82".."arcade_hud: Determines the HUD type. Default Daytona.")
end)

COM_AddCommand("daytonaServerHelp", function(p)
	CONS_Printf(p, "\x82".."arcade: Toggles Arcade Mode on and off. Default On.")
	CONS_Printf(p, "\x82".."arcade_diff: Determines the modifiers for the time limit.")
	CONS_Printf(p, "\x82".."Easiest - 30% extra. Easy - 15% extra. Normal (Default) - default.")
	CONS_Printf(p, "\x82".."Hard - 5s less starting time, 2s less bonus time.")
	CONS_Printf(p, "\x82".."Expert - 10s less starting time, 5s less bonus time.")
	CONS_Printf(p, "\x82".."Master - 20s less starting time, 5s less bonus time.")
	CONS_Printf(p, "\x82".."arcade_sneaker: Limits items to (mainly) Sneakers. Default Off.")
end, 1)